############################################################################
##
#W  buildnew.gi			NQL				Rene Hartung
##
#H   @(#)$Id: buildnew.gi,v 1.3 2007/06/07 13:09:22 gap Exp $
##


############################################################################
##
#F  BuildNewCollector ( )
##
## Build a new collector with attention to the relations and the consistency
## relations for the covering group.
##
InstallGlobalFunction(BuildNewCollector,
  function(Q,ftl,HNF,weights,Defs,Imgs)
  local A,		# power relations with attention to the HNF
	orders,		# relative order of the 'old' collector
	Gens,		# new generators
	QS,		# new quotient system
	rhs,rhs2,	# right hand side of the new relation
	i,j,k,		# loop variables
	c,		# nilpotency class
	b;		# position of the first (pseudo) generator/tail

  # nilpotency class
  c:=Maximum(weights);

  # first new generator/pseudo-generator
  b:=Position(weights,c);

  orders:=RelativeOrders(Q.Pccol);

  # power relations from Hermite normal form
  A:=PowerRelationsOfHNF(HNF);
  
  # find new generators in HNF
  Gens:=HNF.Heads{Filtered([1..Length(HNF.Heads)],
                           x->HNF.mat[x][HNF.Heads[x]]<>1)};
  Append(Gens, Filtered([1..Length(weights)-(b-1)],x->not x in HNF.Heads));
  Sort(Gens);
  
  if Length(Gens)=0 then 
    return(Q);
  fi;
   
  # new quotient system
  QS:=rec();
  QS.Lpres:=Q.Lpres;
  QS.Pccol:=FromTheLeftCollector(b-1+Length(Gens));
  QS.Imgs:=[];
  QS.Definitions:=[];
  QS.Weights:=[];
  
  for i in [1..Length(Defs)] do
    if IsInt(Defs[i]) then 
      if Defs[i]>0 and weights[i]=1 then 
        # generator defined by an image 
        QS.Imgs[Defs[i]]:=Imgs[Defs[i]];
        QS.Weights[i]:=weights[i];
        QS.Definitions[i]:=Defs[i];
      elif Defs[i]>0 and weights[i]>1 then 
        # pseudo-generator from image
        j:=Position(HNF.Heads,i-(b-1));
        if j=fail or HNF.mat[j][i-(b-1)]<>1 then 
          Error("couldn't eliminate pseudo-generator ",i," (imgs)");
        fi;
        QS.Imgs[Defs[i]]:=Concatenation(Imgs[Defs[i]]{[1..Length(
                          Imgs[Defs[i]])-2]},
                          ObjByExponents(QS.Pccol,
                          -Concatenation(ListWithIdenticalEntries(b-1,0),
                                        A[Position(HNF.Heads,i-(b-1))]{Gens})));
      elif Defs[i]<0 then 
        # pseudo-generator from power relation
        j:=Position(HNF.Heads,i-(b-1));
        if j=fail or HNF.mat[j][i-(b-1)]<>1 then 
          Error("couldn't eliminate pseudo-generator ",i-(b-1)," (power rel)");
        fi;
    
        SetRelativeOrder(QS.Pccol,-Defs[i],orders[-Defs[i]]);
        rhs:=GetPower(Q.Pccol,-Defs[i]);
        rhs:=Concatenation(rhs,ObjByExponents(QS.Pccol,
             -Concatenation(ListWithIdenticalEntries(b-1,0),
              A[j]{Gens})));
        SetPower(QS.Pccol,-Defs[i],rhs);
      else
        Error("Strange entry in Defs");
      fi;
    else # commutator as definition
      if weights[i]<c then 
        # (old) generator with definition as COMM ( Defs[i][1], Defs[i][2] )
        rhs:=GetConjugate(Q.Pccol,Defs[i][1],Defs[i][2]);
        QS.Weights[i]:=weights[i];
        QS.Definitions[i]:=Defs[i];
      elif not weights[Defs[i][1]]=c-1 then  
        # pseudo-generator from [a_j,a_i] with w(a_j)<>c-1
        j:=Position(HNF.Heads,i-(b-1));
        if j=fail or HNF.mat[j][i-(b-1)]<>1 then 
          Error("couldn't eliminate pseudo-generator ",i-(b-1)," (comm)");
        fi;
  
        rhs:=GetConjugate(Q.Pccol,Defs[i][1],Defs[i][2]);
        rhs:=Concatenation(rhs,ObjByExponents(QS.Pccol,
             -Concatenation(ListWithIdenticalEntries(b-1,0),
             A[j]{Gens})));
      else
        # (pseudo) generator with [a_j,a_i] and w(a_i)=1
        j:=Position(HNF.Heads,i-(b-1));
        if j=fail then 
          # new (infinite) generator
          k:=i-Length(Filtered([1..Length(HNF.Heads)], 
                               x-> HNF.mat[x][HNF.Heads[x]]=1 
                               and HNF.Heads[x]<i-(b-1)));
          rhs:=[Defs[i][1],1,k,1];
  
          QS.Weights[k]:=weights[i];
          QS.Definitions[k]:=Defs[i];
        elif HNF.mat[j][i-(b-1)]=1 then 
          # generator can be eliminated
          rhs:=GetConjugate(Q.Pccol,Defs[i][1],Defs[i][2]);
          rhs:=Concatenation(rhs,ObjByExponents(QS.Pccol,
               -Concatenation(ListWithIdenticalEntries(b-1,0),
               A[j]{Gens})));
        else
          # new generator with power relation
      	  k:=i-Length(Filtered([1..Length(HNF.Heads)], 
  	  		     x-> HNF.mat[x][HNF.Heads[x]]=1 
                             and HNF.Heads[x]<i-(b-1)));

  	  rhs:=ListWithIdenticalEntries(b-1,0);
	  rhs:=Concatenation(rhs,-A[j]{Gens});
	  rhs[k]:=0;
	  rhs:=ObjByExponents(QS.Pccol,rhs);
		
  	  SetRelativeOrder(QS.Pccol,k,HNF.mat[j][HNF.Heads[j]]);
	  SetPower(QS.Pccol,k,rhs);
		
	  QS.Weights[k]:=weights[i];
	  QS.Definitions[k]:=Defs[i];
	  rhs:=[Defs[i][1],1,k,1];
        fi;
      fi;
      SetConjugate(QS.Pccol,Defs[i][1],Defs[i][2],rhs);
    fi;
  od;

  # remaining commutator relations 
  for i in [1..Length(QS.Weights)-1] do
    for j in [i+1..Length(QS.Weights)] do
      if weights[i]>1 and weights[i]+weights[j]<=c then 
        rhs:=ExponentsByObj(ftl,GetConjugate(ftl,j,i));
        rhs2:=rhs{[NumberOfGenerators(Q.Pccol)+1..NumberOfGenerators(ftl)]};
        rhs:=rhs{[1..NumberOfGenerators(Q.Pccol)]};
        rhs:=Concatenation(rhs,RowReduce(rhs2,HNF){Gens});
        rhs:=ObjByExponents(QS.Pccol,rhs);
        SetConjugate(QS.Pccol,j,i,rhs);
      fi;
    od;
  od;
  UpdatePolycyclicCollector(QS.Pccol);
  
  if TEST_ALL then 
    if not NumberOfGenerators(QS.Pccol)=Length(QS.Weights) or
           Length(QS.Weights)<>Length(QS.Definitions) then 
      Error("wrong Number");
    fi;
  
    if not IsConfluent(QS.Pccol) then
      Error("presentation is not confluent");
    fi;
  fi;
  
  # build the mapping QS.Epimorphism
  Imgs:=[];
  for i in [1..Length(QS.Imgs)] do
    if IsInt(QS.Imgs[i]) then 
      Add(Imgs,PcpElementByGenExpList(QS.Pccol,[QS.Imgs[i],1]));
    else
      Add(Imgs,PcpElementByGenExpList(QS.Pccol,QS.Imgs[i]));
    fi;
  od;
  QS.Epimorphism:=GroupHomomorphismByImagesNC(Q.Lpres, 
  			PcpGroupByCollectorNC(QS.Pccol),
   			GeneratorsOfGroup(Q.Lpres),Imgs);
  
  return(QS);
  end);
