############################################################################
##
#W nq_non.gi			NQL				Rene Hartung
##
#H   @(#)$Id: nq_non.gi,v 1.2 2007/02/28 12:52:21 gap Exp $
##

############################################################################
##
#M NilpotentQuotientLpGroup ( <LpGroup>, <int> ) . . . . for non-invariant LpGroups 
##
## Computes a nilpotent presentation for G/gamma_i(G)
##
InstallMethod( NilpotentQuotientLpGroup,
  "for L-presented groups and positive integer",
  true,
  [ IsLpGroup, IsPosInt ],0,
  function(G,c)
  local InvLp,	# ascending L-presentation
	i,	# loop variable
	Q,	# current quotient system
	QS,	# old quotient system
	NQs;	# nilpotent quotients of <G> (NilpotentQuotients)


  # for G/G' the InitQuotientSystem works for non-invariantly LpGroups too
  NQs:=[];
  Q:=InitQuotientSystem(G);
  NQs[1]:=rec(Quotient:=PcpGroupByCollectorNC(Q.Pccol),
	      Epimorphism:=Q.Epimorphism);
  SetNilpotentQuotients(G,NQs);

  # underlying invariant L-presentation
  InvLp:=UnderlyingInvariantLPresentation(G);
  if not HasIsInvariantLPresentation(InvLp) then 
    SetIsInvariantLPresentation(InvLp,true);
  fi;
  
  # Compute a confluent nilpotent presentation for G/G'
  Q:=InitQuotientSystem(InvLp);

  if Length(Q.Weights)< InfoNQL_MAX_GENS then 
    Info(InfoNQL,1,"Class (non-inv.) ",Maximum(Q.Weights),": ",
			Length(Q.Weights),
			" generators with relative orders: ",
			RelativeOrders(Q.Pccol));
  else
    Info(InfoNQL,1,"Class (non-inv.) ",Maximum(Q.Weights),": ",
			Length(Q.Weights), " generators");
  fi;

  # store the largest nilpotent quotient system of the ascending LpGroup
  SetNilpotentQuotientSystem(InvLp,Q);

  for i in [2..c] do 
    QS:=ShallowCopy(Q);

    # extend the quotient system of G/\gamma_i(G) to G/\gamma_{i+1}(G)
    Q:=ExtendQuotientSystem(Q);

    if not QS.Weights=Q.Weights then 
      if Length(Q.Weights)-Length(QS.Weights)< InfoNQL_MAX_GENS then 
        Info(InfoNQL,1,"Class (non-inv.) ",Maximum(Q.Weights),": ",
			Length(Q.Weights)-Length(QS.Weights),
			" generators with relative orders: ",
			RelativeOrders(Q.Pccol)
			{[Length(QS.Weights)+1..Length(Q.Weights)]});
      else
        Info(InfoNQL,1,"Class (non-inv.) ",Maximum(Q.Weights),": ",
			Length(Q.Weights)-Length(QS.Weights), " generators");
      fi;
    fi;

    if QS.Weights=Q.Weights then 
      # quotient system of the invariant L-presentation terminated
      if not IsBound(NilpotentQuotients(G)[i-1]) then 
        Error("unbound entry in NilpotentQuotients");
      else
	SetLargestNilpotentQuotient(G,NilpotentQuotients(G)[i-1].Quotient);
	Info(InfoNQL,1,"Largest nilpotent quotient of class ", 
			Maximum(Q.Weights));
        return(LargestNilpotentQuotient(G));
      fi;
    elif NQL_TerminatedNonInvariantNQ(G,Q) then 
      if not IsBound(NilpotentQuotients(G)[i-1]) then 
        Error("unbound entry in NilpotentQuotients");
      else
	SetLargestNilpotentQuotient(G,NilpotentQuotients(G)[i-1].Quotient);
        Info(InfoNQL,1,"Largest nilpotent quotient of class ", 
			NilpotencyClassOfGroup(
 			    NilpotentQuotients(G)[i-1].Quotient));
        return(LargestNilpotentQuotient(G));
      fi;
    else 
      # largest quotient system of the invariant L-presentation 
      ResetFilterObj(UnderlyingInvariantLPresentation(G),
			NilpotentQuotientSystem);
      SetNilpotentQuotientSystem(UnderlyingInvariantLPresentation(G),Q);
    fi;
  od;

  return(NilpotentQuotients(G)[c].Quotient); 
  end);

############################################################################
##
#M NilpotentQuotientLpGroup ( <LpGroup>, <int> ) . . . . for non-invariant LpGroups 
##
## Computes a nilpotent presentation for G/gamma_<int>( <LpGroup> )
##
InstallMethod( NilpotentQuotientLpGroup,
  "for L-presented groups (with qs.) and positive integer",
  true,
  [ IsLpGroup and HasNilpotentQuotients, IsPosInt ],0,
  function(G,c)
  local Q,	# largest known quotient system
	i,	# loop variable
	QS,	# smaller quotient system
     	EpiInv,	# epimorphism from the ascending LpGroup into <H>
 	H,	# nilpotent quotient of <InvLp>
        fam,	# family of elements in <G>
	U,	# normal subgroup of the images of the (unit.) relators
     	Epi,MGI,# epimorphism from the LpGroup <G> into <H>/<U>
	NQs,	# nilpotent quotients of <G> (NilpotentQuotients)
 	n, 	# nilpotency class of the largest known quotient system
	nat;	# natural homomorphism <H> -> <H>/<U>

  # check whether this quotient has been computed 
  if IsBound(NilpotentQuotients(G)[c]) then
    return(NilpotentQuotients(G)[c].Quotient);
  fi;

  # restore the largest known quotient system of the underl. inv. L-pres.
  Q:=NilpotentQuotientSystem(UnderlyingInvariantLPresentation(G));

  # nilpotency class of <Q>
  n:=Maximum(Q.Weights);

  NQs:=NilpotentQuotients(G);
  if NilpotencyClassOfGroup(NQs[Length(NQs)].Quotient)<n then 
    Error("may not occur");
#   Info(InfoNQL,1,"Largest nilpotent quotient of class ",
#		NilpotencyClassOfGroup(NQs[Length(NQs)].Quotient));
#   return(NQs[Length(NQs)].Quotient);
  fi;
  
  if n < c then 
    if HasLargestNilpotentQuotient(G) then 
      Info(InfoNQL,1,"Largest nilpotent quotient of class ",
			NilpotencyClassOfGroup(LargestNilpotentQuotient(G)));
      return(LargestNilpotentQuotient(G));
    fi;
    for i in [n+1..c] do
      QS:=ShallowCopy(Q);

      # extend the quotient system of G/\gamma_i(G) to G/\gamma_{i+1}(G)
      Q:=ExtendQuotientSystem(Q);

      if not QS.Weights=Q.Weights then 
        if Length(Q.Weights)-Length(QS.Weights)< InfoNQL_MAX_GENS then 
          Info(InfoNQL,1,"Class (non-inv.) ",Maximum(Q.Weights),": ",
			Length(Q.Weights)-Length(QS.Weights),
			" generators with relative orders: ",
			RelativeOrders(Q.Pccol)
			{[Length(QS.Weights)+1..Length(Q.Weights)]});
        else
          Info(InfoNQL,1,"Class (non-inv.) ",Maximum(Q.Weights),": ",
			Length(Q.Weights)-Length(QS.Weights), " generators");
        fi;
      fi;

      if QS.Weights=Q.Weights then 
        # the quotient system of the ascending presentation terminated
        if not IsBound(NilpotentQuotients(G)[i-1]) then 
          Error("unbound entry in NilpotentQuotients");
        else
 	  Info(InfoNQL,1,"Largest nilpotent quotient of class ",
			  Maximum(Q.Weights));
	  SetLargestNilpotentQuotient(G,NilpotentQuotients(G)[i-1].Quotient);
 	  return(LargestNilpotentQuotient(G));
        fi;
      elif NQL_TerminatedNonInvariantNQ(G,Q) then 
        if not IsBound(NilpotentQuotients(G)[i-1]) then 
          Error("unbound entry in NilpotentQuotients");
        else
 	  Info(InfoNQL,1,"Largest nilpotent quotient of class ",
			  NilpotencyClassOfGroup(
			  NilpotentQuotients(G)[i-1].Quotient));
	  SetLargestNilpotentQuotient(G,NilpotentQuotients(G)[i-1].Quotient);
 	  return(LargestNilpotentQuotient(G));
        fi;
      else
        # largest quotient system of the underl. invariant L-presentation
        ResetFilterObj(UnderlyingInvariantLPresentation(G),
				NilpotentQuotientSystem);
        SetNilpotentQuotientSystem(UnderlyingInvariantLPresentation(G),Q);
      fi;
    od;

    return(NilpotentQuotients(G)[c].Quotient); 
  elif n=c then 
    Error("in NilpotentQuotientLpGroup n=c may not occur");
  else
    # rebuild a smaller quotient system of the underl. inv. L-pres. using <Q>
    QS:=SmallerQuotientSystem(Q,c);
  
    # nilpotent quotient of the invariant L-presentation
    H:=PcpGroupByCollectorNC(QS.Pccol);
  
    # epimorphism from the non-invariant L-presentation into <H> 
    EpiInv:=GroupHomomorphismByImagesNC(G,H,
  		  GeneratorsOfGroup(G),
                  MappingGeneratorsImages(QS.Epimorphism)[2]);

    # relators of <G> are objects of the free group of G
    fam:=ElementsFamily(FamilyObj(G));

    # determine the quotient of the non-invariant L-presentation
    U:=NormalClosure(H,Subgroup(H,List(FixedRelatorsOfLpGroup(G),
      	  			x->ElementOfLpGroup(fam,x)^EpiInv)));

    # natural homomorphism H -> H/U
    nat:=NaturalHomomorphism(H,U);

    # the epimorphism G -> H/U
    MGI:=MappingGeneratorsImages(EpiInv*nat);
    Epi:=GroupHomomorphismByImagesNC(G,Image(nat),MGI[1],MGI[2]);

    # store the epimorphism and the nilpotent quotient in NilpotentQuotients
    NQs:=ShallowCopy(NilpotentQuotients(G));
    NQs[c]:=rec(Quotient:=Image(nat),
  	        Epimorphism:=Epi);
    ResetFilterObj(G,NilpotentQuotients);
    SetNilpotentQuotients(G,NQs);

    return(NQs[c].Quotient); 
  fi;
  end);


############################################################################
##
#M NilpotentQuotientLpGroup( <LpGroup> ) . . . . . . . . for non-invariant LpGroups
##
## determines the largest nilpotent quotient of an L-presented group
## Attention: this method may not terminate!
##
InstallOtherMethod( NilpotentQuotientLpGroup,
  "for L-presented groups",
  true,
  [ IsLpGroup ],0,
  function(G)
  local InvLp,	# ascending L-presentation
        Q,	# current quotient system
	QS,	# old quotient system
     	EpiInv,	# epimorphism from the ascending LpGroup into <H>
	H,	# nilpotent quotient of <InvLp>
        fam,	# family of elements in <G>
	U,	# normal subgroup of the images of the (unit.) relators
  	nat,	# natural homomorphism <H> -> <H>/<U>
     	Epi,MGI,# epimorphism from the ascending LpGroup into <H>/<U>
      	c,	# nilpotency class of the largest nilpotent quotient 
        NQs;	# nilpotent quotients with epimorphisms (NilpotentQuotients)

  
  # for G/G' the InitQuotientSystem works for non-invariantly LpGroups, too
  NQs:=[];
  Q:=InitQuotientSystem(G);
  NQs[1]:=rec(Quotient:=PcpGroupByCollectorNC(Q.Pccol),
	      Epimorphism:=Q.Epimorphism);
  SetNilpotentQuotients(G,NQs);

  # the corresponding ascending presentation
  InvLp:=UnderlyingInvariantLPresentation(G);
  if not HasIsInvariantLPresentation(InvLp) then 
    SetIsInvariantLPresentation(InvLp,true);
  fi;

  # nilpotent presentation for <InvLp>/<InvLp>'
  Q:=InitQuotientSystem(InvLp);

  if Length(Q.Weights)< InfoNQL_MAX_GENS then 
    Info(InfoNQL,1,"Class (non-inv.) ",Maximum(Q.Weights),": ",
			Length(Q.Weights),
			" generators with relative orders: ",
			RelativeOrders(Q.Pccol));
  else
    Info(InfoNQL,1,"Class (non-inv.) ",Maximum(Q.Weights),": ",
			Length(Q.Weights), " generators");
  fi;

  # store the largest quotient system of the invariant LpGroup <InvLp>
  SetNilpotentQuotientSystem(InvLp,Q);

  repeat 
    QS:=ShallowCopy(Q);

    # extend the quotient system of G/\gamma_i(G) to G/\gamma_{i+1}(G)
    Q:=ExtendQuotientSystem(Q);
  
    if not QS.Weights=Q.Weights then 
      if Length(Q.Weights)-Length(QS.Weights)< InfoNQL_MAX_GENS then 
        Info(InfoNQL,1,"Class (non-inv.) ",Maximum(Q.Weights),": ",
			Length(Q.Weights)-Length(QS.Weights),
			" generators with relative orders: ",
			RelativeOrders(Q.Pccol)
			{[Length(QS.Weights)+1..Length(Q.Weights)]});
      else
        Info(InfoNQL,1,"Class (non-inv.) ",Maximum(Q.Weights),": ",
			Length(Q.Weights)-Length(QS.Weights), " generators");
      fi;
    fi;
    
    if NQL_TerminatedNonInvariantNQ(G,Q) then 
      Info(InfoNQL,1,"Largest nilpotent quotient of class ",
		      NilpotencyClassOfGroup(
		      NilpotentQuotients(G)[Maximum(Q.Weights)-1].Quotient));
      SetLargestNilpotentQuotient(G,NilpotentQuotients(G)
					 [Maximum(Q.Weights)-1].Quotient);
      return(LargestNilpotentQuotient(G));
    fi;

  until QS.Weights=Q.Weights;

  Info(InfoNQL,1,"Largest nilpotent quotient of class ", 
		  Maximum(Q.Weights));

  SetLargestNilpotentQuotient(G,NilpotentQuotients(G)
 					[Maximum(Q.Weights)].Quotient);
  return(LargestNilpotentQuotient(G));
  end);

############################################################################
##
#M NilpotentQuotientLpGroup( <LpGroup> ) . . . . . . . . for non-invariant LpGroups
##
## determines the largest nilpotent quotient of an L-presented group
## Attention: this method may not terminate!
##
InstallOtherMethod( NilpotentQuotientLpGroup,
  "for L-presented groups (with qs)",
  true,
  [ IsLpGroup and HasNilpotentQuotients ],0,
  function(G)
  local Q,	# current quotient system
	QS,	# old quotient system
        NQs;	# nilpotent quotients with epimorphisms (NilpotentQuotients)

  # the largest nilpotent quotient has been computed
  if HasLargestNilpotentQuotient(G) then
    Info(InfoNQL,1,"Largest nilpotent quotient of class ",
 		NilpotencyClassOfGroup(LargestNilpotentQuotient(G)));
    return(LargestNilpotentQuotient(G));
  fi;

  # largest known nilpotent quotient system of the ascending presentation
  Q:=NilpotentQuotientSystem(UnderlyingInvariantLPresentation(G));

  # is the nilpotency class of the largest quotient system too big?
  NQs:=NilpotentQuotients(G);
  if NilpotencyClassOfGroup(NQs[Length(NQs)].Quotient)<Maximum(Q.Weights) then 
    Error("may not occur");
#   Info(InfoNQL,1,"Largest nilpotent quotient of class ",
#		NilpotencyClassOfGroup(NQs[Length(NQs)].Quotient));
#   return(NQs[Length(NQs)].Quotient);
  fi;

  repeat 
    QS:=ShallowCopy(Q);

    # extend the quotient system of G/\gamma_i(G) to G/\gamma_{i+1}(G)
    Q:=ExtendQuotientSystem(Q);

    if not QS.Weights=Q.Weights then 
      if Length(Q.Weights)-Length(QS.Weights)< InfoNQL_MAX_GENS then 
        Info(InfoNQL,1,"Class (non-inv.) ",Maximum(Q.Weights),": ",
			Length(Q.Weights)-Length(QS.Weights),
			" generators with relative orders: ",
			RelativeOrders(Q.Pccol)
			{[Length(QS.Weights)+1..Length(Q.Weights)]});
      else
        Info(InfoNQL,1,"Class (non-inv.) ",Maximum(Q.Weights),": ",
			Length(Q.Weights)-Length(QS.Weights), " generators");
      fi;
    fi;

    if NQL_TerminatedNonInvariantNQ(G,Q) then 
      Info(InfoNQL,1,"Largest nilpotent quotient of class ",
 		     NilpotencyClassOfGroup(
		     NilpotentQuotients(G)[Maximum(Q.Weights)-1].Quotient));
      SetLargestNilpotentQuotient(G,NilpotentQuotients(G)
  					[Maximum(Q.Weights)-1].Quotient);
      return(LargestNilpotentQuotient(G));
    fi;
  
  until QS.Weights=Q.Weights;
  
  Info(InfoNQL,1,"Largest nilpotent quotient of class ", 
		  Maximum(Q.Weights));

  SetLargestNilpotentQuotient(G,NilpotentQuotients(G)
				[Maximum(Q.Weights)].Quotient);
  return(LargestNilpotentQuotient(G));
  end);

############################################################################
##
#M  NqEpimorphismNilpotentQuotientLpGroup ( <LpGroup> )
##
## Computes an epimorphism from the LpGroup into the largest nilpotent
## quotient if it exists.
##
InstallOtherMethod( NqEpimorphismNilpotentQuotientLpGroup,
  "for L-presented groups",
  true,  
  [ IsLpGroup, IsPosInt ], 0,
  function(G,c)
  local H;	# nilpotent quotient of G

  H:=NilpotentQuotientLpGroup(G,c);   
  return(NilpotentQuotients(G)[c].Epimorphism);
  end);

############################################################################
##
#M  NqEpimorphismNilpotentQuotientLpGroup ( <LpGroup> )
##
## Computes an epimorphism from the LpGroup into the largest nilpotent
## quotient if it exists.
##
InstallOtherMethod( NqEpimorphismNilpotentQuotientLpGroup,
  "for L-presented groups",
  true,  
  [ IsLpGroup and HasNilpotentQuotients, IsPosInt ], 0,
  function(G,c)
  local Q,	# largest known nilpotent quotient system of <G>
	H, 	# nilpotent quotient of <G>
        n,	# nilpotency class of the largest nilp. qs. of <G>
 	NQs;	# all known nilpotent quotients of <G>

  if IsBound(NilpotentQuotients(G)[c]) then 
    return(NilpotentQuotients(G)[c].Epimorphism);
  else
    H:=NilpotentQuotientLpGroup(G,c);
    return(NilpotentQuotients(G)[c].Epimorphism);
  fi;
  end);

############################################################################
##
#M  NqEpimorphismNilpotentQuotientLpGroup ( <LpGroup> )
##
## Computes an epimorphism from the LpGroup into the largest nilpotent
## quotient if it exists.
##
InstallOtherMethod( NqEpimorphismNilpotentQuotientLpGroup,
  "for L-presented groups",
  true,  
  [ IsLpGroup and HasNilpotentQuotients, IsPcpGroup ], 0,
  function(G,H)
  local i, 	# loop variable
	NQs;	# all known nilpotent quotients of <G>

  # largest known nilpotent quotient system of <G>
  NQs:=NilpotentQuotients(G);

  for i in [1..Length(NQs)] do
    if IsBound(NQs[i]) and NQs[i].Quotient=H then 
       return(NQs[i].Epimorphism);
    fi;
  od;
  Error("<H> must be nilpotent quotient of <G>");
  end);

############################################################################
##
#M  NqEpimorphismNilpotentQuotientLpGroup ( <LpGroup> )
##
## Computes an epimorphism from the LpGroup into the largest nilpotent
## quotient if it exists.
##
InstallOtherMethod( NqEpimorphismNilpotentQuotientLpGroup,
  "for L-presented groups",
  true,  
  [ IsLpGroup ], 0,
  function(G)
  local H,	# nilpotent quotient of G
	c;	# nilpotency class of the largest nilpotent quotient

  c:=NilpotencyClassOfGroup(LargestNilpotentQuotient(G));
  
  return(NilpotentQuotients(G)[c].Epimorphism);
  end);


############################################################################
##
#M  NqEpimorphismNilpotentQuotientLpGroup ( <LpGroup> )
##
## Computes an epimorphism from the LpGroup into the largest nilpotent
## quotient if it exists and a nilpotent quotient system is already known.
##
InstallOtherMethod( NqEpimorphismNilpotentQuotientLpGroup,
  "for L-presented groups",
  true,  
  [ IsLpGroup and HasNilpotentQuotients ], 0,
  function(G)
  local c;	# nilpotency class of the largest nilpotent quotient 

  c:=NilpotencyClassOfGroup(LargestNilpotentQuotient(G));   
  return(NilpotentQuotients(G)[c].Epimorphism);
  end);

############################################################################
##
#F  NQL_TerminatedNonInvariantNQ( <LpGroup>, <QS> )
##
InstallGlobalFunction( NQL_TerminatedNonInvariantNQ,
  function(G,Q)
  local H,	# nilpotent quotient of the invariant LpGroup
	EpiInv,	# epimorphism G->H
	fam,	# family of LpGroup-elements
	U,	# normal subgroup of H generated by the images of the fixed rels
	nat,	# natural homomorphism H->H/U
	c,	# nilpotency class of <Q>
	MGI,	# mapping generators images of EpiInv*nat
	Epi,	# epimorphism G->H/U
	NQs;	# new NilpotentQuotients

   # largest quotient system of the invariant L-presentation 
   ResetFilterObj(UnderlyingInvariantLPresentation(G),NilpotentQuotientSystem);
   SetNilpotentQuotientSystem(UnderlyingInvariantLPresentation(G),Q);

   # nilpotent quotient of the ascending L-presentation
   H:=PcpGroupByCollectorNC(Q.Pccol);

   # the epimorphism from the non-ascending presentation into <H> 
   EpiInv:=GroupHomomorphismByImagesNC(G,H,
  		  GeneratorsOfGroup(G),
                  MappingGeneratorsImages(Q.Epimorphism)[2]);
    
   # relators of G are objects of the free group of G
   fam:=ElementsFamily(FamilyObj(G));
  
   # determine the quotient of the non-ascending presentation
   U:=NormalClosure(H,Subgroup(H,List(FixedRelatorsOfLpGroup(G),
       	  			x->ElementOfLpGroup(fam,x)^EpiInv)));
  
   # natural homomorphism H -> H/U
   nat:=NaturalHomomorphism(H,U);

   # nilpotency class of <Q>
   c:=Maximum(Q.Weights);
  
   if NilpotencyClassOfGroup( Image(nat) ) < c then 
     return(true); 
   fi;

   # the epimorphism G -> H/U
   MGI:=MappingGeneratorsImages(EpiInv*nat);
   Epi:=GroupHomomorphismByImagesNC(G,Image(nat),MGI[1],MGI[2]);
 
   # store the epimorphism and the nilpotent quotient in NilpotentQuotients
   NQs:=ShallowCopy(NilpotentQuotients(G));
   NQs[c]:=rec(Quotient:=Image(nat), Epimorphism:=Epi);
   ResetFilterObj(G,NilpotentQuotients);
   SetNilpotentQuotients(G,NQs);

   return(false);
   end);

